@@ -1,7 +1,7 @@
 package Sidef {
 
     use 5.014;
-    our $VERSION = '2.34';
+    our $VERSION = '2.35';
 
     our $SPACES      = 0;    # the current number of spaces
     our $SPACES_INCR = 4;    # the number of spaces incrementor
@@ -427,7 +427,7 @@ package Sidef {
 };
 
 #
-## Some UNIVERSAL magic
+## UNIVERSAL methods
 #
 
 *UNIVERSAL::get_value = sub {
@@ -455,10 +455,7 @@ package Sidef {
         }
 
         if (defined(&$AUTOLOAD)) {
-            goto \&$AUTOLOAD;
-
-            ## The above `goto` is roughly equivalent with:
-            #return $AUTOLOAD->($self, @args);
+            goto &$AUTOLOAD;
         }
     }
 
