@@ -397,10 +397,8 @@ HEADER
             ',',
             grep { $_ ne '' } map {
                 (ref($_) eq 'Sidef::Types::String::String' or ref($_) eq 'Sidef::Types::Number::Number')
-                  ? $self->_dump_string($_->get_value)
-                  : ref($_) ? ('(map { ref($_) eq "Sidef::Types::String::String" ? $$_ '
-                               . ': do {state$sub;$sub=UNIVERSAL::can($_, "..."); '
-                               . 'defined($sub) ? $sub->($_) : "$_" } } '
+                  ? $self->_dump_string("$_")
+                  : ref($_) ?    ('(map { ref($_) eq "Sidef::Types::String::String" ? $$_ : "$_" }'
                                . ($self->deparse_expr(ref($_) eq 'HASH' ? $_ : {self => $_})) . ')')
                   : qq{"\Q$_\E"}
               } @{$array}
@@ -545,7 +543,7 @@ HEADER
                     my $name = $obj->{name};
 
                     # Check for alphanumeric name
-                    if (not $obj->{name} =~ /^[_\pL][_\pL\pN]*\z/) {
+                    if (not $obj->{name} =~ /^[_\pL][\pL\pN\w]*\z/) {
                         $obj->{name} = '__NONANN__';    # use this name for non-alphanumeric names
                     }
 
@@ -700,14 +698,14 @@ HEADER
                     $] < 5.025002 && $self->top_add(q{use feature 'lexical_subs'; no warnings 'experimental::lexical_subs';});
 
                     # XXX: this is known to cause segmentation faults in perl-5.18.* and perl-5.20.* when used in a class
-                    $code = "state sub $name(){state\$_$refaddr"
+                    $code = "my sub $name(){state\$_$refaddr"
                       . (defined($obj->{expr}) ? ('=do{' . $self->deparse_script($obj->{expr}) . '}') : '') . '}';
                 }
 
                 # Otherwise, use static constants
                 else {
                     $code = "sub $name(){state\$_$refaddr"
-                      . (defined($obj->{expr}) ? '=do{' . ($self->deparse_script($obj->{expr}) . '}') : '') . '}';
+                      . (defined($obj->{expr}) ? ('=do{' . $self->deparse_script($obj->{expr}) . '}') : '') . '}';
                 }
             }
             else {
@@ -1165,6 +1163,8 @@ HEADER
             $code = 'continue';
         }
         elsif ($ref eq 'Sidef::Types::Block::Return') {
+
+            # This is no longer supported
             if (not exists $expr->{call}) {
 
                 if (exists $self->{function}) {
