@@ -11,27 +11,24 @@ package Sidef::Object::Object {
       q{~~}   => \&{__PACKAGE__ . '::' . '~~'},
       q{bool} => sub {
         if (defined(my $sub = UNIVERSAL::can($_[0], 'to_b'))) {
-            $sub->($_[0]);
-        }
-        else {
-            $_[0];
+            @_ = ($_[0]);
+            goto $sub;
         }
+        $_[0];
       },
       q{0+} => sub {
         if (defined(my $sub = UNIVERSAL::can($_[0], 'to_n'))) {
-            $sub->($_[0]);
-        }
-        else {
-            $_[0];
+            @_ = ($_[0]);
+            goto $sub;
         }
+        $_[0];
       },
       q{""} => sub {
         if (defined(my $sub = UNIVERSAL::can($_[0], 'to_s'))) {
-            $sub->($_[0]);
-        }
-        else {
-            $_[0];
+            @_ = ($_[0]);
+            goto $sub;
         }
+        $_[0];
       },
       q{cmp} => sub {
         my ($obj1, $obj2, $swaped) = @_;
@@ -47,7 +44,8 @@ package Sidef::Object::Object {
         if (   CORE::ref($obj1) && UNIVERSAL::isa($obj1, CORE::ref($obj2))
             or CORE::ref($obj2) && UNIVERSAL::isa($obj2, CORE::ref($obj1))) {
             if (defined(my $sub = UNIVERSAL::can($obj1, '<=>'))) {
-                return $sub->($obj1, $obj2);
+                @_ = ($obj1, $obj2);
+                goto $sub;
             }
         }
 
@@ -70,61 +68,13 @@ package Sidef::Object::Object {
           || return (Sidef::Types::Bool::Bool::FALSE);
 
         if (defined(my $sub = UNIVERSAL::can($obj1, '=='))) {
-            return $sub->($obj1, $obj2);
+            @_ = ($obj1, $obj2);
+            goto $sub;
         }
 
-        ($obj1 cmp $obj2)->is_zero;
+        !CORE::int($obj1 cmp $obj2);
       };
 
-    # Alternative way for comparing cyclic references
-    # (quite slow -- we don't use it)
-#<<<
-    #~ sub __eq__ {
-        #~ my %addr;
-
-        #~ my $sub = sub {
-            #~ my ($obj1, $obj2) = @_;
-
-            #~ my $refaddr1 = Scalar::Util::refaddr($obj1);
-            #~ my $refaddr2 = Scalar::Util::refaddr($obj2);
-
-            #~ if ($refaddr1 > $refaddr2) {
-                #~ ($refaddr1, $refaddr2) = ($refaddr2, $refaddr1);
-            #~ }
-
-            #~ (   $obj1->SUPER::isa(CORE::ref($obj2) || return (Sidef::Types::Bool::Bool::FALSE))
-             #~ || $obj2->SUPER::isa(CORE::ref($obj1) || return (Sidef::Types::Bool::Bool::FALSE)))
-              #~ || return (Sidef::Types::Bool::Bool::FALSE);
-
-            #~ return Sidef::Types::Bool::Bool::TRUE
-              #~ if ($refaddr1 == $refaddr2);
-
-            #~ if (defined(my $sub = $obj1->can('=='))) {
-
-                #~ my $key = join(' ', $refaddr1, $refaddr2);
-
-                #~ exists($addr{$key})
-                  #~ && return $addr{$key};
-
-                #~ my $bool = 1;
-                #~ my $bool_obj = bless \$bool, 'Sidef::Types::Bool::Bool';
-
-                #~ $addr{$key} = $bool_obj;
-                #~ $bool = $sub->($obj1, $obj2);
-                #~ return $bool_obj;
-            #~ }
-
-            #~ ($obj1 cmp $obj2)->is_zero;
-        #~ };
-
-        #~ eval 'use overload "eq" => $sub;';
-        #~ my $result = $sub->(@_);
-        #~ eval 'use overload "eq" => \&__eq__;';
-
-        #~ $result;
-    #~ }
-#>>>
-
     sub new {
         bless {}, __PACKAGE__;
     }
@@ -160,6 +110,13 @@ package Sidef::Object::Object {
 
     *refaddr = \&object_id;
 
+    sub object_type {
+        my ($self) = @_;
+        Sidef::Types::String::String->new(Scalar::Util::reftype($self));
+    }
+
+    *reftype = \&object_type;
+
     sub class {
         my ($obj) = @_;
         my $ref = CORE::ref($obj) || $obj;
