@@ -67,7 +67,7 @@ package Sidef::Types::Array::Array {
         my @arg  = @$arg;
         my @self = @$self;
 
-        (my $argc = @arg) || return $self->new(\@self);
+        (my $argc = @arg) || return bless(\@self, __PACKAGE__);
         my $selfc = @self;
 
         my $max = $argc > $selfc ? $argc - 1 : $selfc - 1;
@@ -76,7 +76,7 @@ package Sidef::Types::Array::Array {
             CORE::push(@array, $self[$i % $selfc]->$operator($arg[$i % $argc]));
         }
 
-        $self->new(\@array);
+        bless \@array, __PACKAGE__;
     }
 
     sub map_operator {
@@ -89,7 +89,7 @@ package Sidef::Types::Array::Array {
             CORE::push(@array, $self->[$i]->$operator(@args));
         }
 
-        $self->new(\@array);
+        bless \@array, __PACKAGE__;
     }
 
     sub pam_operator {
@@ -102,14 +102,14 @@ package Sidef::Types::Array::Array {
             CORE::push(@array, $arg->$operator($self->[$i]));
         }
 
-        $self->new(\@array);
+        bless \@array, __PACKAGE__;
     }
 
     sub reduce_operator {
         my ($self, $operator) = @_;
 
         $operator = "$operator" if ref($operator);
-        (my $end = $#$self) >= 0 || return;
+        (my $end = $#$self) >= 0 || return undef;
 
         my $x = $self->[0];
         foreach my $i (1 .. $end) {
@@ -129,7 +129,7 @@ package Sidef::Types::Array::Array {
         if ($operator eq '') {
             foreach my $i (@$self) {
                 foreach my $j (@arg) {
-                    CORE::push(@array, $self->new([$i, $j]));
+                    CORE::push(@array, bless([$i, $j], __PACKAGE__));
                 }
             }
         }
@@ -141,7 +141,7 @@ package Sidef::Types::Array::Array {
             }
         }
 
-        $self->new(\@array);
+        bless \@array, __PACKAGE__;
     }
 
     sub zip_operator {
@@ -159,7 +159,7 @@ package Sidef::Types::Array::Array {
         my @array;
         if ($operator eq '') {
             foreach my $i (0 .. $min) {
-                CORE::push(@array, $self->new([$self[$i], $arg[$i]]));
+                CORE::push(@array, bless([$self[$i], $arg[$i]], __PACKAGE__));
             }
         }
         else {
@@ -168,12 +168,12 @@ package Sidef::Types::Array::Array {
             }
         }
 
-        $self->new(\@array);
+        bless \@array, __PACKAGE__;
     }
 
     sub mul {
         my ($self, $num) = @_;
-        $self->new([(@$self) x $num]);
+        bless [(@$self) x $num], __PACKAGE__;
     }
 
     sub div {
@@ -189,12 +189,12 @@ package Sidef::Types::Array::Array {
         while (@obj) {
             my $j = $pos - $i * CORE::int($len);
             $pos -= $j if $j >= 1;
-            CORE::push(@array, $self->new([splice @obj, 0, $len + $j]));
+            CORE::push(@array, bless [splice @obj, 0, $len + $j], __PACKAGE__);
             $pos += $len;
             $i++;
         }
 
-        $self->new(\@array);
+        bless \@array, __PACKAGE__;
     }
 
     sub part {
@@ -203,7 +203,7 @@ package Sidef::Types::Array::Array {
         my @first = @$self;
         my @second = splice(@first, CORE::int($num));
 
-        ($self->new(\@first), $self->new(\@second));
+        (bless(\@first, __PACKAGE__), bless(\@second, __PACKAGE__));
     }
 
     *partition = \&part;
@@ -231,13 +231,13 @@ package Sidef::Types::Array::Array {
 
         while (1) {
 
-            $cmp = $x[$i] cmp $y[$j];
+            $cmp = CORE::int($x[$i] cmp $y[$j]);
 
-            if (ref($cmp) ? $cmp->is_mone : ($cmp == -1)) {
+            if ($cmp < 0) {
                 push @new, $x[$i];
                 ++$i;
             }
-            elsif (ref($cmp) ? $cmp->is_one : ($cmp == 1)) {
+            elsif ($cmp > 0) {
                 push @new, $y[$j];
                 ++$j;
             }
@@ -257,7 +257,7 @@ package Sidef::Types::Array::Array {
             }
         }
 
-        return $self->new(\@new);
+        bless \@new, __PACKAGE__;
     }
 
     sub and {
@@ -275,12 +275,12 @@ package Sidef::Types::Array::Array {
         my ($cmp, @new);
         while ($i < $end1 and $j < $end2) {
 
-            $cmp = $x[$i] cmp $y[$j];
+            $cmp = CORE::int($x[$i] cmp $y[$j]);
 
-            if (ref($cmp) ? $cmp->is_mone : ($cmp == -1)) {
+            if ($cmp < 0) {
                 ++$i;
             }
-            elsif (ref($cmp) ? $cmp->is_one : ($cmp == 1)) {
+            elsif ($cmp > 0) {
                 ++$j;
             }
             else {
@@ -290,7 +290,7 @@ package Sidef::Types::Array::Array {
             }
         }
 
-        $self->new(\@new);
+        bless \@new, __PACKAGE__;
     }
 
     sub is_empty {
@@ -313,13 +313,13 @@ package Sidef::Types::Array::Array {
         my ($cmp, @new);
         while ($i < $end1 and $j < $end2) {
 
-            $cmp = $x[$i] cmp $y[$j];
+            $cmp = CORE::int($x[$i] cmp $y[$j]);
 
-            if (ref($cmp) ? $cmp->is_mone : ($cmp == -1)) {
+            if ($cmp < 0) {
                 push @new, $x[$i];
                 ++$i;
             }
-            elsif (ref($cmp) ? $cmp->is_one : ($cmp == 1)) {
+            elsif ($cmp > 0) {
                 ++$j;
             }
             else {
@@ -331,15 +331,15 @@ package Sidef::Types::Array::Array {
             push @new, @x[$i .. $#x];
         }
 
-        $self->new(\@new);
+        bless \@new, __PACKAGE__;
     }
 
     sub concat {
         my ($self, $arg) = @_;
 
         UNIVERSAL::isa($arg, 'ARRAY')
-          ? $self->new([@$self, @$arg])
-          : $self->new([@$self, $arg]);
+          ? bless([@$self, @$arg], __PACKAGE__)
+          : bless([@$self, $arg],  __PACKAGE__);
     }
 
     sub levenshtein {
@@ -425,60 +425,6 @@ package Sidef::Types::Array::Array {
         Sidef::Types::Number::Number->new($jaro + $prefix * 0.1 * (1 - $jaro));
     }
 
-    sub combinations {
-        my ($self, $k, $block) = @_;
-
-        $k = CORE::int($k);
-
-        if (defined($block)) {
-
-            if ($k == 0) {
-                $block->run($self->new);
-                return $self;
-            }
-
-            return if $k < 0;
-
-            my $n = @$self;
-            return $self if ($k > $n or $n == 0);
-
-            my @c = (0 .. $k - 1);
-
-            while (1) {
-                $block->run($self->new([@$self[@c]]));
-                next if ($c[$k - 1]++ < $n - 1);
-                my $i = $k - 2;
-                $i-- while ($i >= 0 && $c[$i] >= $n - ($k - $i));
-                last if $i < 0;
-                $c[$i]++;
-                while (++$i < $k) { $c[$i] = $c[$i - 1] + 1; }
-            }
-
-            return $self;
-        }
-
-        return $self->new($self->new) if $k == 0;
-        return if $k < 0;
-
-        my $n = @$self;
-        return $self->new if ($k > $n or $n == 0);
-
-        my @c = (0 .. $k - 1);
-        my @result;
-
-        while (1) {
-            CORE::push(@result, $self->new(@$self[@c]));
-            next if ($c[$k - 1]++ < $n - 1);
-            my $i = $k - 2;
-            $i-- while ($i >= 0 && $c[$i] >= $n - ($k - $i));
-            last if $i < 0;
-            $c[$i]++;
-            while (++$i < $k) { $c[$i] = $c[$i - 1] + 1; }
-        }
-
-        $self->new(\@result);
-    }
-
     sub count {
         my ($self, $obj) = @_;
 
@@ -535,10 +481,10 @@ package Sidef::Types::Array::Array {
                 CORE::push(@tmp_array, $array->[$i]);
             }
 
-            CORE::push(@new_array, $self->new(\@tmp_array));
+            CORE::push(@new_array, bless(\@tmp_array, __PACKAGE__));
         }
 
-        $self->new(\@new_array);
+        bless(\@new_array, __PACKAGE__);
     }
 
     sub mzip {
@@ -551,34 +497,34 @@ package Sidef::Types::Array::Array {
             CORE::push(@new_array, $self->[$i], $array->[$i % $arr_max]);
         }
 
-        $self->new(\@new_array);
+        bless(\@new_array, __PACKAGE__);
     }
 
     sub make {
         my ($self, $size, $type) = @_;
-        $self->new([($type) x $size]);
+        bless([($type) x $size], __PACKAGE__);
     }
 
     sub _min_max {
         my ($self, $value) = @_;
 
-        @$self || return;
+        @$self || return undef;
 
         my $item = $self->[0];
         foreach my $i (1 .. $#$self) {
             my $val = $self->[$i];
-            $item = $val if (($val cmp $item) eq $value);
+            $item = $val if (CORE::int($val cmp $item) == $value);
         }
 
         $item;
     }
 
     sub max {
-        $_[0]->_min_max(Sidef::Types::Number::Number::ONE);
+        $_[0]->_min_max(1);
     }
 
     sub min {
-        $_[0]->_min_max(Sidef::Types::Number::Number::MONE);
+        $_[0]->_min_max(-1);
     }
 
     sub minmax {
@@ -590,18 +536,23 @@ package Sidef::Types::Array::Array {
         $_[0]->reduce_operator('+');
     }
 
+    sub sum_by {
+        my ($self, $arg) = @_;
+
+        my $sum = Sidef::Types::Number::Number::ZERO;
+
+        foreach my $obj (@$self) {
+            $sum = $sum->add($arg->run($obj));
+        }
+
+        return $sum;
+    }
+
     sub sum {
         my ($self, $arg) = @_;
 
-        # Sum by mapping each value to a block
         if (ref($arg) eq 'Sidef::Types::Block::Block') {
-            my $sum = Sidef::Types::Number::Number::ZERO;
-
-            foreach my $obj (@$self) {
-                $sum = $sum->add($arg->run($obj));
-            }
-
-            return $sum;
+            goto \&sum_by;
         }
 
         my $sum = $arg // Sidef::Types::Number::Number::ZERO;
@@ -613,18 +564,23 @@ package Sidef::Types::Array::Array {
         $sum;
     }
 
+    sub prod_by {
+        my ($self, $arg) = @_;
+
+        my $prod = Sidef::Types::Number::Number::ONE;
+
+        foreach my $obj (@$self) {
+            $prod = $prod->mul($arg->run($obj));
+        }
+
+        return $prod;
+    }
+
     sub prod {
         my ($self, $arg) = @_;
 
-        # Product by mapping each value to a block
         if (ref($arg) eq 'Sidef::Types::Block::Block') {
-            my $prod = Sidef::Types::Number::Number::ONE;
-
-            foreach my $obj (@$self) {
-                $prod = $prod->mul($arg->run($obj));
-            }
-
-            return $prod;
+            goto \&prod_by;
         }
 
         my $prod = $arg // Sidef::Types::Number::Number::ONE;
@@ -639,24 +595,26 @@ package Sidef::Types::Array::Array {
     sub _min_max_by {
         my ($self, $code, $value) = @_;
 
-        @$self || return;
+        @$self || return undef;
 
         my @pairs = map { [$_, scalar $code->run($_)] } @$self;
         my $item = $pairs[0];
 
         foreach my $i (1 .. $#pairs) {
-            $item = $pairs[$i] if (($pairs[$i][1] cmp $item->[1]) eq $value);
+            $item = $pairs[$i] if (CORE::int($pairs[$i][1] cmp $item->[1]) == $value);
         }
 
         $item->[0];
     }
 
     sub max_by {
-        $_[0]->_min_max_by($_[1], Sidef::Types::Number::Number::ONE);
+        push @_, 1;
+        goto \&_min_max_by;
     }
 
     sub min_by {
-        $_[0]->_min_max_by($_[1], Sidef::Types::Number::Number::MONE);
+        push @_, -1;
+        goto \&_min_max_by;
     }
 
     sub swap {
@@ -677,12 +635,12 @@ package Sidef::Types::Array::Array {
         if (defined $arg) {
 
             if (ref($arg) eq 'Sidef::Types::Block::Block') {
-                return $self->first_by($arg);
+                goto \&first_by;
             }
 
             my $max = $#$self;
             $arg = CORE::int($arg) - 1;
-            return $self->new(@$self[0 .. ($arg > $max ? $max : $arg)]);
+            return bless([@$self[0 .. ($arg > $max ? $max : $arg)]], __PACKAGE__);
         }
 
         @$self ? $self->[0] : ();
@@ -694,11 +652,11 @@ package Sidef::Types::Array::Array {
         if (defined $arg) {
 
             if (ref($arg) eq 'Sidef::Types::Block::Block') {
-                return $self->last_by($arg);
+                goto \&last_by;
             }
 
             my $from = @$self - CORE::int($arg);
-            return $self->new([@$self[($from < 0 ? 0 : $from) .. $#$self]]);
+            return bless([@$self[($from < 0 ? 0 : $from) .. $#$self]], __PACKAGE__);
         }
 
         @$self ? $self->[-1] : ();
@@ -725,7 +683,7 @@ package Sidef::Types::Array::Array {
             CORE::push(@new_array, ref($item) eq ref($self) ? ($item->_flatten) : $item);
         }
 
-        $self->new(\@new_array);
+        bless \@new_array, __PACKAGE__;
     }
 
     *flat = \&flatten;
@@ -748,7 +706,7 @@ package Sidef::Types::Array::Array {
 
     sub items {
         my ($self, @indices) = @_;
-        $self->new([map { exists($self->[$_]) ? $self->[$_] : undef } @indices]);
+        bless([map { exists($self->[$_]) ? $self->[$_] : undef } @indices], __PACKAGE__);
     }
 
     sub item {
@@ -764,10 +722,10 @@ package Sidef::Types::Array::Array {
     sub dig {
         my ($self, $key, @keys) = @_;
 
-        my $value = $self->fetch($key) // return;
+        my $value = $self->fetch($key) // return undef;
 
         foreach my $key (@keys) {
-            $value = $value->fetch($key) // return;
+            $value = $value->fetch($key) // return undef;
         }
 
         $value;
@@ -802,7 +760,7 @@ package Sidef::Types::Array::Array {
 
     sub ft {
         my ($self) = @_;
-        $self->new([_slice(@_)]);
+        bless [_slice(@_)], __PACKAGE__;
     }
 
     *slice = \&ft;
@@ -827,12 +785,12 @@ package Sidef::Types::Array::Array {
 
         my $end = @$self;
         for (my $i = $n - 1 ; $i < $end ; $i += $n) {
-            $code->run($self->new([@$self[$i - ($n - 1) .. $i]]));
+            $code->run(bless([@$self[$i - ($n - 1) .. $i]], __PACKAGE__));
         }
 
         my $mod = $end % $n;
         if ($mod != 0) {
-            $code->run($self->new([@$self[$end - $mod .. $end - 1]]));
+            $code->run(bless([@$self[$end - $mod .. $end - 1]], __PACKAGE__));
         }
 
         $self;
@@ -850,7 +808,7 @@ package Sidef::Types::Array::Array {
             push @{$new[$i]}, $item;
         }
 
-        $self->new([map { $self->new($_) } @new]);
+        bless([map { bless($_, __PACKAGE__) } @new], __PACKAGE__);
     }
 
     sub slice_after {
@@ -860,12 +818,10 @@ package Sidef::Types::Array::Array {
         my $i = 0;
         foreach my $item (@$self) {
             push @{$new[$i]}, $item;
-            if ($block->run($item)) {
-                ++$i;
-            }
+            ++$i if $block->run($item);
         }
 
-        $self->new([map { $self->new($_) } @new]);
+        bless [map { bless($_, __PACKAGE__) } @new], __PACKAGE__;
     }
 
     sub slices {
@@ -873,18 +829,34 @@ package Sidef::Types::Array::Array {
 
         $n = CORE::int($n);
 
+        $n <= 0
+          && return bless([], __PACKAGE__);
+
         my @slices;
         my $end = @$self;
         for (my $i = $n - 1 ; $i < $end ; $i += $n) {
-            CORE::push(@slices, $self->new([@$self[$i - ($n - 1) .. $i]]));
+            CORE::push(@slices, bless([@$self[$i - ($n - 1) .. $i]], __PACKAGE__));
         }
 
         my $mod = $end % $n;
         if ($mod != 0) {
-            CORE::push(@slices, $self->new([@$self[$end - $mod .. $end - 1]]));
+            CORE::push(@slices, bless([@$self[$end - $mod .. $end - 1]], __PACKAGE__));
         }
 
-        $self->new(\@slices);
+        bless \@slices, __PACKAGE__;
+    }
+
+    sub cons {
+        my ($self, $n) = @_;
+
+        $n = CORE::int($n);
+
+        my @array;
+        foreach my $i ($n - 1 .. $#$self) {
+            push @array, bless([@$self[$i - $n + 1 .. $i]], __PACKAGE__);
+        }
+
+        bless \@array, __PACKAGE__;
     }
 
     sub each_cons {
@@ -893,7 +865,7 @@ package Sidef::Types::Array::Array {
         $n = CORE::int($n);
 
         foreach my $i ($n - 1 .. $#$self) {
-            $code->run($self->new([@$self[$i - $n + 1 .. $i]]));
+            $code->run(bless([@$self[$i - $n + 1 .. $i]], __PACKAGE__));
         }
 
         $self;
@@ -929,7 +901,7 @@ package Sidef::Types::Array::Array {
             CORE::push(@array, $code->run($item));
         }
 
-        $self->new(\@array);
+        bless \@array, __PACKAGE__;
     }
 
     sub map_kv {
@@ -940,7 +912,7 @@ package Sidef::Types::Array::Array {
             CORE::push(@arr, $code->run(Sidef::Types::Number::Number->_set_uint($i), $self->[$i]));
         }
 
-        $self->new(\@arr);
+        bless \@arr, __PACKAGE__;
     }
 
     sub flat_map {
@@ -951,7 +923,7 @@ package Sidef::Types::Array::Array {
             CORE::push(@array, @{scalar $code->run($item)});
         }
 
-        $self->new(\@array);
+        bless \@array, __PACKAGE__;
     }
 
     sub grep {
@@ -969,7 +941,7 @@ package Sidef::Types::Array::Array {
             }
         }
 
-        $self->new(\@array);
+        bless \@array, __PACKAGE__;
     }
 
     *select = \&grep;
@@ -977,12 +949,12 @@ package Sidef::Types::Array::Array {
     sub grep_kv {
         my ($self, $code) = @_;
 
-        my @arr;
+        my @array;
         foreach my $i (0 .. $#$self) {
-            CORE::push(@arr, $self->[$i]) if $code->run(Sidef::Types::Number::Number->_set_uint($i), $self->[$i]);
+            CORE::push(@array, $self->[$i]) if $code->run(Sidef::Types::Number::Number->_set_uint($i), $self->[$i]);
         }
 
-        $self->new(\@arr);
+        bless \@array, __PACKAGE__;
     }
 
     *select_kv = \&grep_kv;
@@ -990,15 +962,15 @@ package Sidef::Types::Array::Array {
     sub group_by {
         my ($self, $code) = @_;
 
-        my $hash = Sidef::Types::Hash::Hash->new;
+        my %hash;
         foreach my $item (@$self) {
             my $key = $code->run(my $val = $item);
             my $str_key = "$key";
-            exists($hash->{$str_key}) or do { $hash->{$str_key} = $self->new };
-            CORE::push(@{$hash->{$str_key}}, $val);
+            exists($hash{$str_key}) or do { $hash{$str_key} = bless([], __PACKAGE__) };
+            CORE::push(@{$hash{$str_key}}, $val);
         }
 
-        $hash;
+        Sidef::Types::Hash::Hash->new(%hash);
     }
 
     sub match {
@@ -1044,25 +1016,6 @@ package Sidef::Types::Array::Array {
         );
     }
 
-    sub freq {
-        my ($self, $code) = @_;
-
-        if (defined($code)) {
-            return $self->freq_by($code);
-        }
-
-        my %hash;
-        foreach my $item (@$self) {
-            $hash{$item}++;
-        }
-
-        foreach my $key (keys %hash) {
-            $hash{$key} = Sidef::Types::Number::Number->_set_uint($hash{$key});
-        }
-
-        Sidef::Types::Hash::Hash->new(%hash);
-    }
-
     sub freq_by {
         my ($self, $code) = @_;
 
@@ -1078,12 +1031,31 @@ package Sidef::Types::Array::Array {
         Sidef::Types::Hash::Hash->new(%hash);
     }
 
+    sub freq {
+        my ($self, $code) = @_;
+
+        if (defined($code)) {
+            goto \&freq_by;
+        }
+
+        my %hash;
+        foreach my $item (@$self) {
+            $hash{$item}++;
+        }
+
+        foreach my $key (keys %hash) {
+            $hash{$key} = Sidef::Types::Number::Number->_set_uint($hash{$key});
+        }
+
+        Sidef::Types::Hash::Hash->new(%hash);
+    }
+
     sub first_by {
         my ($self, $code) = @_;
         foreach my $val (@$self) {
             return $val if $code->run($val);
         }
-        return;
+        return undef;
     }
 
     *find = \&first_by;
@@ -1093,7 +1065,7 @@ package Sidef::Types::Array::Array {
         for (my $i = $#$self ; $i >= 0 ; --$i) {
             return $self->[$i] if $code->run($self->[$i]);
         }
-        return;
+        return undef;
     }
 
     sub any {
@@ -1123,15 +1095,17 @@ package Sidef::Types::Array::Array {
     sub assign_to {
         my ($self, @vars) = @_;
 
-        my @values = splice(@$self, 0, $#vars + 1);
+        my @values = CORE::splice(@$self, 0, $#vars + 1);
 
         for my $i (0 .. $#vars) {
             if (exists $values[$i]) {
-                ${$vars[$i]} = $values[$i];
+                if (ref($vars[$i]) eq 'REF') {
+                    ${$vars[$i]} = $values[$i];
+                }
             }
         }
 
-        $self->new(\@values);
+        bless \@values, __PACKAGE__;
     }
 
     sub bindex {
@@ -1146,15 +1120,14 @@ package Sidef::Types::Array::Array {
             while ($left <= $right) {
                 $middle = (($right + $left) >> 1);
                 $item   = $self->[$middle];
-                $cmp    = $obj->run($item) || return Sidef::Types::Number::Number->_set_uint($middle);
+                $cmp    = CORE::int($obj->run($item)) || return Sidef::Types::Number::Number->_set_uint($middle);
 
-                if ($cmp eq Sidef::Types::Number::Number::ONE) {
+                if ($cmp > 0) {
                     $right = $middle - 1;
                 }
                 else {
                     $left = $middle + 1;
                 }
-
             }
 
             return Sidef::Types::Number::Number::MONE;
@@ -1163,12 +1136,12 @@ package Sidef::Types::Array::Array {
         while ($left <= $right) {
             $middle = int(($right + $left) >> 1);
             $item   = $self->[$middle];
-            $cmp    = $item cmp $obj;
+            $cmp    = CORE::int($item cmp $obj);
 
             if (!$cmp) {
                 return Sidef::Types::Number::Number->_set_uint($middle);
             }
-            elsif ($cmp eq Sidef::Types::Number::Number::ONE) {
+            elsif ($cmp > 0) {
                 $right = $middle - 1;
             }
             else {
@@ -1240,19 +1213,19 @@ package Sidef::Types::Array::Array {
     sub pairmap {
         my ($self, $obj) = @_;
 
-        my $end = @$self || return $self->new;
+        my $end = @$self;
 
         my @array;
         for (my $i = 1 ; $i < $end ; $i += 2) {
             CORE::push(@array, scalar $obj->run(@$self[$i - 1, $i]));
         }
 
-        $self->new(\@array);
+        bless \@array, __PACKAGE__;
     }
 
     sub shuffle {
         my ($self) = @_;
-        $self->new([List::Util::shuffle(@$self)]);
+        bless [List::Util::shuffle(@$self)], __PACKAGE__;
     }
 
     sub best_shuffle {
@@ -1287,7 +1260,7 @@ package Sidef::Types::Array::Array {
         my ($self, $obj, $initial) = @_;
 
         if (ref($obj) eq 'Sidef::Types::Block::Block') {
-            (my $end = $#$self) >= 0 || return;
+            (my $end = $#$self) >= 0 || return undef;
             my ($beg, $x) = defined($initial) ? (0, $initial) : (1, $self->[0]);
             foreach my $i ($beg .. $end) {
                 $x = $obj->run($x, $self->[$i]);
@@ -1328,19 +1301,49 @@ package Sidef::Types::Array::Array {
 
     *resize_to = \&resize;
 
+    sub pick {
+        my ($self, $amount) = @_;
+
+        if (defined $amount) {
+            $amount = CORE::int($amount);
+
+            my $len = @$self;
+            if ($amount >= $len) {
+                return bless([List::Util::shuffle(@$self)], __PACKAGE__);
+            }
+
+            my @result;
+            for (my ($i, $amount_left) = (0, $len) ; $amount > 0 ; ++$i, --$amount_left) {
+                my $rand = CORE::int(CORE::rand($amount_left));
+                if ($rand < $amount) {
+                    push @result, $self->[$i];
+                    $amount--;
+                }
+            }
+
+            return bless([List::Util::shuffle(@result)], __PACKAGE__);
+        }
+
+        $self->[CORE::rand(scalar @$self)];
+    }
+
     sub rand {
         my ($self, $amount) = @_;
 
         if (defined $amount) {
             $amount = CORE::int($amount);
-            return @$self
-              ? $self->new([map { $self->[CORE::rand(scalar @$self)] } 1 .. $amount])
-              : Sidef::Types::Array::Array->new([]);
+
+            my $len = @$self;
+            return (
+                    $len > 0
+                    ? bless([map { $self->[CORE::rand($len)] } 1 .. $amount], __PACKAGE__)
+                    : bless([], __PACKAGE__)
+                   );
         }
+
         $self->[CORE::rand(scalar @$self)];
     }
 
-    *pick   = \&rand;
     *sample = \&rand;
 
     sub range {
@@ -1353,15 +1356,15 @@ package Sidef::Types::Array::Array {
 
     sub indices {
         my ($self) = @_;
-        $self->new([map { Sidef::Types::Number::Number->_set_uint($_) } 0 .. $#$self]);
+        bless [map { Sidef::Types::Number::Number->_set_uint($_) } 0 .. $#$self], __PACKAGE__;
     }
 
     *keys = \&indices;
 
     sub pairs {
         my ($self) = @_;
-        $self->new(
-              [map { Sidef::Types::Array::Pair->new(Sidef::Types::Number::Number->_set_uint($_), $self->[$_]) } 0 .. $#$self]);
+        bless [map { Sidef::Types::Array::Pair->new(Sidef::Types::Number::Number->_set_uint($_), $self->[$_]) } 0 .. $#$self],
+          __PACKAGE__;
     }
 
     *kv = \&pairs;
@@ -1394,9 +1397,9 @@ package Sidef::Types::Array::Array {
         while (1) {
             $middle = (($right + $left) >> 1);
             $item   = $self->[$middle];
-            $cmp    = ($item cmp $obj) || last;
+            $cmp    = CORE::int($item cmp $obj) || last;
 
-            if ($cmp eq Sidef::Types::Number::Number::MONE) {
+            if ($cmp < 0) {
                 $left = $middle + 1;
                 if ($left > $right) {
                     ++$middle;
@@ -1415,7 +1418,7 @@ package Sidef::Types::Array::Array {
 
     sub compact {
         my ($self) = @_;
-        $self->new([grep { defined($_) } @$self]);
+        bless([grep { defined($_) } @$self], __PACKAGE__);
     }
 
     sub unique {
@@ -1437,7 +1440,7 @@ package Sidef::Types::Array::Array {
             ++$i while ($i < $max and $sorted[$i][1] eq $sorted[$i + 1][1]);
         }
 
-        $self->new([grep { defined } @unique]);
+        bless [grep { defined } @unique], __PACKAGE__;
     }
 
     *uniq     = \&unique;
@@ -1462,7 +1465,7 @@ package Sidef::Types::Array::Array {
             $unique[$sorted[$i][0]] = $sorted[$i][1];
         }
 
-        $self->new([grep { defined } @unique]);
+        bless [grep { defined } @unique], __PACKAGE__;
     }
 
     *last_uniq = \&last_unique;
@@ -1487,7 +1490,7 @@ package Sidef::Types::Array::Array {
             ++$i while ($i < $max and $sorted[$i][2] eq $sorted[$i + 1][2]);
         }
 
-        $self->new([grep { defined } @unique]);
+        bless [grep { defined } @unique], __PACKAGE__;
     }
 
     *unique_by = \&uniq_by;
@@ -1512,7 +1515,7 @@ package Sidef::Types::Array::Array {
             $unique[$sorted[$i][0]] = $sorted[$i][1];
         }
 
-        $self->new([grep { defined } @unique]);
+        bless [grep { defined } @unique], __PACKAGE__;
     }
 
     *last_unique_by = \&last_uniq_by;
@@ -1547,10 +1550,10 @@ package Sidef::Types::Array::Array {
                         if ($key eq $tail) {
 
                             if ($callback) {
-                                $block->run($self->new([@{$ref->{$key}}[0 .. $#{$ref->{$key}} - $count]]));
+                                $block->run(bless([@{$ref->{$key}}[0 .. $#{$ref->{$key}} - $count]], __PACKAGE__));
                             }
                             else {
-                                CORE::push(@abbrev, $self->new([@{$ref->{$key}}[0 .. $#{$ref->{$key}} - $count]]));
+                                CORE::push(@abbrev, bless([@{$ref->{$key}}[0 .. $#{$ref->{$key}} - $count]], __PACKAGE__));
                             }
 
                             last;
@@ -1563,7 +1566,7 @@ package Sidef::Types::Array::Array {
           }
           ->(\%table);
 
-        $self->new(\@abbrev);
+        bless \@abbrev, __PACKAGE__;
     }
 
     *abbreviations = \&abbrev;
@@ -1636,10 +1639,9 @@ package Sidef::Types::Array::Array {
         my ($self, $num) = @_;
 
         if (defined $num) {
-            return $self->new([CORE::splice(@$self, 0, $num,)]);
+            return bless([CORE::splice(@$self, 0, $num)], __PACKAGE__);
         }
 
-        @$self || return;
         shift(@$self);
     }
 
@@ -1652,10 +1654,9 @@ package Sidef::Types::Array::Array {
         if (defined $num) {
             $num = CORE::int($num);
             $num = $num > $#$self ? 0 : @$self - $num;
-            return $self->new([CORE::splice(@$self, $num)]);
+            return bless([CORE::splice(@$self, $num)], __PACKAGE__);
         }
 
-        @$self || return;
         pop(@$self);
     }
 
@@ -1664,7 +1665,7 @@ package Sidef::Types::Array::Array {
 
     sub pop_rand {
         my ($self) = @_;
-        $#$self > -1 || return;
+        $#$self > -1 || return undef;
         CORE::splice(@$self, CORE::rand(scalar @$self), 1);
     }
 
@@ -1682,7 +1683,7 @@ package Sidef::Types::Array::Array {
         $offset = defined($offset) ? CORE::int($offset) : 0;
         $length = defined($length) ? CORE::int($length) : scalar(@$self);
 
-        $self->new([CORE::splice(@$self, $offset, $length, @objects)]);
+        bless([CORE::splice(@$self, $offset, $length, @objects)], __PACKAGE__);
     }
 
     sub take_right {
@@ -1692,7 +1693,7 @@ package Sidef::Types::Array::Array {
         $amount = CORE::int($amount);
         $amount = $end > ($amount - 1) ? $amount - 1 : $end;
 
-        $self->new([@$self[$end - $amount .. $end]]);
+        bless [@$self[$end - $amount .. $end]], __PACKAGE__;
     }
 
     sub take_left {
@@ -1702,22 +1703,22 @@ package Sidef::Types::Array::Array {
         $amount = CORE::int($amount);
         $amount = $end > ($amount - 1) ? $amount - 1 : $end;
 
-        $self->new([@$self[0 .. $amount]]);
+        bless [@$self[0 .. $amount]], __PACKAGE__;
     }
 
     sub sort {
         my ($self, $code) = @_;
 
         if (defined $code) {
-            return $self->new([CORE::sort { scalar $code->run($a, $b) } @$self]);
+            return bless([CORE::sort { scalar $code->run($a, $b) } @$self], __PACKAGE__);
         }
 
-        $self->new([CORE::sort { $a cmp $b } @$self]);
+        bless [CORE::sort { $a cmp $b } @$self], __PACKAGE__;
     }
 
     sub sort_by {
         my ($self, $code) = @_;
-        $self->new([map { $_->[0] } sort { $a->[1] cmp $b->[1] } map { [$_, scalar $code->run($_)] } @$self]);
+        bless [map { $_->[0] } sort { $a->[1] cmp $b->[1] } map { [$_, scalar $code->run($_)] } @$self], __PACKAGE__;
     }
 
     sub cmp {
@@ -1726,20 +1727,12 @@ package Sidef::Types::Array::Array {
         my $l1 = $#$self;
         my $l2 = $#$arg;
 
+        my $cmp;
         my $min = $l1 < $l2 ? $l1 : $l2;
 
         foreach my $i (0 .. $min) {
-
-            my $obj1 = $self->[$i];
-            my $obj2 = $arg->[$i];
-
-            my $value = $obj1 cmp $obj2;
-
-            if (ref($value)) {
-                $value->is_zero or return $value;
-            }
-            elsif ($value != 0) {
-                return ($value < 0 ? Sidef::Types::Number::Number::MONE : Sidef::Types::Number::Number::ONE);
+            if ($cmp = CORE::int($self->[$i] cmp $arg->[$i])) {
+                return ($cmp < 0 ? Sidef::Types::Number::Number::MONE : Sidef::Types::Number::Number::ONE);
             }
         }
 
@@ -1748,34 +1741,104 @@ package Sidef::Types::Array::Array {
           :              Sidef::Types::Number::Number::ONE;
     }
 
-    # Insert an object between each element
+    # Inserts an object between each element
     sub join_insert {
         my ($self, $delim_obj) = @_;
 
-        @$self || return $self->new;
+        my $end = $#$self;
+
+        $end >= 0
+          || return bless([], __PACKAGE__);
 
         my @array = $self->[0];
-        foreach my $i (1 .. $#$self) {
+        foreach my $i (1 .. $end) {
             CORE::push(@array, $delim_obj, $self->[$i]);
         }
-        $self->new(\@array);
+
+        bless \@array, __PACKAGE__;
     }
 
-    sub permute {
+    sub combinations {
+        my ($self, $k, $block) = @_;
+
+        $k = CORE::int($k);
+
+        if (defined($block)) {
+
+            if ($k == 0) {
+                $block->run(bless [], __PACKAGE__);
+                return $self;
+            }
+
+            my $n = @$self;
+            return $self if ($k < 0 or $k > $n or $n == 0);
+
+            my @c = (0 .. $k - 1);
+
+            while (1) {
+                $block->run(bless [@$self[@c]], __PACKAGE__);
+                next if ($c[$k - 1]++ < $n - 1);
+                my $i = $k - 2;
+                $i-- while ($i >= 0 && $c[$i] >= $n - ($k - $i));
+                last if $i < 0;
+                $c[$i]++;
+                while (++$i < $k) { $c[$i] = $c[$i - 1] + 1; }
+            }
+
+            return $self;
+        }
+
+        ($k == 0)
+          && return bless [bless [], __PACKAGE__], __PACKAGE__;
+
+        my $n = @$self;
+
+        ($k < 0 or $k > $n or $n == 0)
+          && return bless([], __PACKAGE__);
+
+        my @c = (0 .. $k - 1);
+        my @result;
+
+        while (1) {
+            CORE::push(@result, bless([@$self[@c]], __PACKAGE__));
+            next if ($c[$k - 1]++ < $n - 1);
+            my $i = $k - 2;
+            $i-- while ($i >= 0 && $c[$i] >= $n - ($k - $i));
+            last if $i < 0;
+            $c[$i]++;
+            while (++$i < $k) { $c[$i] = $c[$i - 1] + 1; }
+        }
+
+        bless \@result, __PACKAGE__;
+    }
+
+    *each_comb = \&combinations;
+
+    sub permutations {
         my ($self, $code) = @_;
 
-        @$self || return $self;
         my @idx = 0 .. $#$self;
 
+        if (not @idx) {
+
+            if (defined $code) {
+                $code->run(bless [], __PACKAGE__);
+                return $self;
+            }
+
+            return bless [bless [], __PACKAGE__], __PACKAGE__;
+        }
+
         if (defined($code)) {
             my $perm;
+
             while (1) {
-                $perm = $self->new([@$self[@idx]]);
+                $perm = bless([@$self[@idx]], __PACKAGE__);
 
                 my $p = $#idx;
                 --$p while $idx[$p - 1] > $idx[$p];
 
-                my $q = $p or do {
+                my $q = $p || do {
                     $code->run($perm);
                     return $self;
                 };
@@ -1787,23 +1850,24 @@ package Sidef::Types::Array::Array {
                 $code->run($perm);
             }
 
-            return;
+            return $self;
         }
 
         my @array;
         while (1) {
-            CORE::push(@array, $self->new([@$self[@idx]]));
+            CORE::push(@array, bless([@$self[@idx]], __PACKAGE__));
             my $p = $#idx;
             --$p while $idx[$p - 1] > $idx[$p];
-            my $q = $p or (return $self->new(\@array));
+            my $q = $p || (return bless(\@array, __PACKAGE__));
             CORE::push(@idx, CORE::reverse CORE::splice @idx, $p);
             ++$q while $idx[$p - 1] > $idx[$q];
             @idx[$p - 1, $q] = @idx[$q, $p - 1];
         }
     }
 
-    *permutations = \&permute;
-    *permutation  = \&permute;
+    *permute     = \&permutations;    # deprecated
+    *permutation = \&permutations;    # deprecated
+    *each_perm   = \&permutations;
 
     sub pack {
         my ($self, $format) = @_;
@@ -1816,6 +1880,7 @@ package Sidef::Types::Array::Array {
         $self;
     }
 
+    *add    = \&push;
     *append = \&push;
 
     sub unshift {
@@ -1831,15 +1896,11 @@ package Sidef::Types::Array::Array {
 
         $num = CORE::int($num);
         $num %= ($#$self + 1);
-        return $self->new([@$self]) if $num == 0;
+        return bless([@$self], __PACKAGE__) if $num == 0;
 
-        # Surprisingly, this is slower:
-        # $self->new(@$self[$num .. $#$self], @$self[0 .. $num - 1]);
-
-        # Surprisingly, this is 73% faster:
         my @array = @$self;
         CORE::unshift(@array, CORE::splice(@array, $num));
-        $self->new(\@array);
+        bless \@array, __PACKAGE__;
     }
 
     # Join the array as string
@@ -1869,7 +1930,7 @@ package Sidef::Types::Array::Array {
 
     sub reverse {
         my ($self) = @_;
-        $self->new([CORE::reverse @$self]);
+        bless [CORE::reverse @$self], __PACKAGE__;
     }
 
     *flip = \&reverse;
@@ -1897,8 +1958,7 @@ package Sidef::Types::Array::Array {
         my ($self, $obj) = @_;
 
         foreach my $i (0 .. $#$self) {
-            my $item = $self->[$i];
-            if ($item eq $obj) {
+            if ($self->[$i] eq $obj) {
                 CORE::splice(@$self, $i, 1);
                 return (Sidef::Types::Bool::Bool::TRUE);
             }
@@ -1913,8 +1973,7 @@ package Sidef::Types::Array::Array {
         my ($self, $obj) = @_;
 
         for (my $i = $#$self ; $i >= 0 ; $i--) {
-            my $item = $self->[$i];
-            if ($item eq $obj) {
+            if ($self->[$i] eq $obj) {
                 CORE::splice(@$self, $i, 1);
                 return (Sidef::Types::Bool::Bool::TRUE);
             }
@@ -1928,10 +1987,9 @@ package Sidef::Types::Array::Array {
     sub delete {
         my ($self, $obj) = @_;
 
-        for (my $i = 0 ; $i <= $#$self ; $i++) {
-            my $item = $self->[$i];
-            if ($item eq $obj) {
-                CORE::splice(@$self, $i--, 1);
+        for (my $i = $#$self ; $i >= 0 ; --$i) {
+            if ($self->[$i] eq $obj) {
+                CORE::splice(@$self, $i, 1);
             }
         }
 
@@ -1944,8 +2002,9 @@ package Sidef::Types::Array::Array {
         my ($self, $code) = @_;
 
         for (my $i = 0 ; $i <= $#$self ; $i++) {
-            $code->run($self->[$i])
-              && CORE::splice(@$self, $i--, 1);
+            if ($code->run($self->[$i])) {
+                CORE::splice(@$self, $i--, 1);
+            }
         }
 
         $self;
@@ -1957,11 +2016,10 @@ package Sidef::Types::Array::Array {
         my ($self, $code) = @_;
 
         foreach my $i (0 .. $#$self) {
-            my $item = $self->[$i];
-            $code->run($item) && do {
+            if ($code->run($self->[$i])) {
                 CORE::splice(@$self, $i, 1);
                 return (Sidef::Types::Bool::Bool::TRUE);
-            };
+            }
         }
 
         (Sidef::Types::Bool::Bool::FALSE);
@@ -1974,10 +2032,10 @@ package Sidef::Types::Array::Array {
 
         for (my $i = $#$self ; $i >= 0 ; --$i) {
             my $item = $self->[$i];
-            $code->run($item) && do {
+            if ($code->run($item)) {
                 CORE::splice(@$self, $i, 1);
                 return (Sidef::Types::Bool::Bool::TRUE);
-              }
+            }
         }
 
         (Sidef::Types::Bool::Bool::FALSE);
@@ -2056,11 +2114,10 @@ package Sidef::Types::Array::Array {
             }
         }
 
-        $self->new([map { Sidef::Types::String::String->new($_) } @argv]);
+        bless [map { Sidef::Types::String::String->new($_) } @argv], __PACKAGE__;
     }
 
     sub _dump {
-
         my %addr;    # keeps track of dumped objects
 
         my $sub = sub {
@@ -2089,10 +2146,7 @@ package Sidef::Types::Array::Array {
         Sidef::Types::String::String->new($_[0]->_dump);
     }
 
-    sub to_s {
-        my ($self) = @_;
-        Sidef::Types::String::String->new(CORE::join(' ', @$self));
-    }
+    *to_s = \&dump;
 
     {
         no strict 'refs';
