@@ -65,12 +65,17 @@ package Sidef::Types::Number::Complex {
         }
 
         if (not defined($y)) {
+
+            if (ref($x) eq 'Math::MPC') {
+                return bless \$x, __PACKAGE__;
+            }
+
             my $r = Math::MPC::Rmpc_init2($PREC);
             if (ref($x) eq 'Math::GMPq' or ref($x) eq 'SCALAR') {
                 Math::MPC::Rmpc_set_q($r, $x, $ROUND);
             }
             else {
-                Math::MPC::Rmpc_set_str($r, $x, 10, $ROUND);
+                Math::MPC::Rmpc_set_str($r, (defined($x) ? "$x" : 0), 10, $ROUND);
             }
 
             return (bless \$r, __PACKAGE__);
@@ -114,21 +119,21 @@ package Sidef::Types::Number::Complex {
             }
             else {
                 my $y_fr = Math::MPFR::Rmpfr_init2($PREC);
-                Math::MPFR::Rmpfr_set_str($y_fr, $y, 10, $Sidef::Types::Number::Number::ROUND);
+                Math::MPFR::Rmpfr_set_str($y_fr, "$y", 10, $Sidef::Types::Number::Number::ROUND);
                 Math::MPC::Rmpc_set_q_fr($r, $x, $y_fr, $ROUND);
             }
         }
         elsif (ref($y) eq 'Math::GMPq' or ref($y) eq 'SCALAR') {
             my $x_fr = Math::MPFR::Rmpfr_init2($PREC);
-            Math::MPFR::Rmpfr_set_str($x_fr, $x, 10, $Sidef::Types::Number::Number::ROUND);
+            Math::MPFR::Rmpfr_set_str($x_fr, "$x", 10, $Sidef::Types::Number::Number::ROUND);
             Math::MPC::Rmpc_set_fr_q($r, $x_fr, $y, $ROUND);
         }
         else {
             my $x_fr = Math::MPFR::Rmpfr_init2($PREC);
-            Math::MPFR::Rmpfr_set_str($x_fr, $x, 10, $Sidef::Types::Number::Number::ROUND);
+            Math::MPFR::Rmpfr_set_str($x_fr, "$x", 10, $Sidef::Types::Number::Number::ROUND);
 
             my $y_fr = Math::MPFR::Rmpfr_init2($PREC);
-            Math::MPFR::Rmpfr_set_str($y_fr, $y, 10, $Sidef::Types::Number::Number::ROUND);
+            Math::MPFR::Rmpfr_set_str($y_fr, "$y", 10, $Sidef::Types::Number::Number::ROUND);
 
             Math::MPC::Rmpc_set_fr_fr($r, $x_fr, $y_fr, $ROUND);
 
@@ -395,6 +400,13 @@ package Sidef::Types::Number::Complex {
 
     *fpow = \&pow;
 
+    sub sqr {
+        my ($x) = @_;
+        my $r = Math::MPC::Rmpc_init2($PREC);
+        Math::MPC::Rmpc_sqr($r, $$x, $ROUND);
+        bless(\$r, __PACKAGE__);
+    }
+
     sub root {
         my ($x, $y) = @_;
         return $x->pow($y->inv);
@@ -502,7 +514,10 @@ package Sidef::Types::Number::Complex {
         Math::MPC::Rmpc_log($d, $c, $ROUND);
 
         my $x = Math::MPC::Rmpc_init2($PREC);
-        Math::MPC::Rmpc_set_ui($x, 1, $ROUND);
+        Math::MPC::Rmpc_set($x, $c, $ROUND);
+        Math::MPC::Rmpc_sqr($x, $x, $ROUND);
+        Math::MPC::Rmpc_add_ui($x, $x, 1, $ROUND);
+        Math::MPC::Rmpc_log($x, $x, $ROUND);
 
         my $y = Math::MPC::Rmpc_init2($PREC);
         Math::MPC::Rmpc_set_ui($y, 0, $ROUND);
@@ -527,6 +542,51 @@ package Sidef::Types::Number::Complex {
             last if ++$count > $PREC;
         }
 
+        #~ say "Complex.lgrt(): $count";
+
+        bless \$x, __PACKAGE__;
+    }
+
+    sub lambert_w {
+        my $c = ${$_[0]};
+
+        $PREC = CORE::int($PREC);
+
+        my $p = Math::MPFR::Rmpfr_init2($PREC);
+        Math::MPFR::Rmpfr_ui_pow_ui($p, 10, CORE::int($PREC / 4), $Sidef::Types::Number::Number::ROUND);
+        Math::MPFR::Rmpfr_ui_div($p, 1, $p, $Sidef::Types::Number::Number::ROUND);
+
+        my $x = Math::MPC::Rmpc_init2($PREC);
+        Math::MPC::Rmpc_set($x, $c, $ROUND);
+        Math::MPC::Rmpc_sqrt($x, $x, $ROUND);
+        Math::MPC::Rmpc_add_ui($x, $x, 1, $ROUND);
+
+        my $y = Math::MPC::Rmpc_init2($PREC);
+        Math::MPC::Rmpc_set_ui($y, 0, $ROUND);
+
+        my $tmp = Math::MPC::Rmpc_init2($PREC);
+        my $abs = Math::MPFR::Rmpfr_init2($PREC);
+
+        my $count = 0;
+        while (1) {
+            Math::MPC::Rmpc_sub($tmp, $x, $y, $ROUND);
+
+            Math::MPC::Rmpc_abs($abs, $tmp, $ROUND);
+            Math::MPFR::Rmpfr_cmp($abs, $p) <= 0 and last;
+
+            Math::MPC::Rmpc_set($y, $x, $ROUND);
+
+            Math::MPC::Rmpc_log($tmp, $x, $ROUND);
+            Math::MPC::Rmpc_add_ui($tmp, $tmp, 1, $ROUND);
+
+            Math::MPC::Rmpc_add($x, $x, $c, $ROUND);
+            Math::MPC::Rmpc_div($x, $x, $tmp, $ROUND);
+            last if ++$count > $PREC;
+        }
+
+        #~ say "Complex.lambert_w(): $count";
+
+        Math::MPC::Rmpc_log($x, $x, $ROUND);
         bless \$x, __PACKAGE__;
     }
 
@@ -814,6 +874,13 @@ package Sidef::Types::Number::Complex {
         bless(\$r, __PACKAGE__);
     }
 
+    sub sin_cos {
+        my $cos = Math::MPC::Rmpc_init2($PREC);
+        my $sin = Math::MPC::Rmpc_init2($PREC);
+        Math::MPC::Rmpc_sin_cos($sin, $cos, ${$_[0]}, $ROUND, $ROUND);
+        (bless(\$sin, __PACKAGE__), bless(\$cos, __PACKAGE__));
+    }
+
     #
     ## Testing
     #
@@ -929,7 +996,7 @@ package Sidef::Types::Number::Complex {
 
     sub round {
         my ($x, $prec) = @_;
-        $x->abs->round($prec);
+        $x->abs->round(defined($prec) ? $prec : ());
     }
 
     *roundf = \&round;
