@@ -147,6 +147,7 @@ package Sidef::Types::Number::Inf {
     *rad               = \&nan;
     *bessel_j          = \&nan;
     *bessel_y          = \&nan;
+    *beta              = \&nan;
 
     *harm     = \&inf;
     *harmonic = \&inf;
@@ -216,9 +217,13 @@ package Sidef::Types::Number::Inf {
     *cbrt             = \&inf;
     *sqr              = \&inf;
     *log              = \&inf;
+    *ilog             = \&inf;
     *lgrt             = \&inf;
+    *lambert_w        = \&inf;
     *log2             = \&inf;
+    *ilog2            = \&inf;
     *log10            = \&inf;
+    *ilog10           = \&inf;
     *exp              = \&inf;
     *exp2             = \&inf;
     *exp10            = \&inf;
@@ -340,6 +345,10 @@ package Sidef::Types::Number::Inf {
         state $x = Sidef::Types::Number::Complex->new((Sidef::Types::Number::Nan::NAN) x 2);
     }
 
+    sub sin_cos {
+        ((Sidef::Types::Number::Nan::NAN) x 2);
+    }
+
     #
     ## binomial(inf, x) = 0       | with x < 0
     ## binomial(inf, inf) = 1
@@ -516,9 +525,16 @@ package Sidef::Types::Number::Inf {
 
     *nude = \&parts;
 
-    sub seed { }
+    sub seed { undef; }
+
     *iseed = \&seed;
 
+    *forperm = \&seed;
+    *forcomb = \&seed;
+
+    *permutations = \&seed;
+    *combinations = \&seed;
+
     {
         no strict 'refs';
         *{__PACKAGE__ . '::' . '+'}   = \&add;
