@@ -72,9 +72,13 @@ package Sidef::Types::Number::Nan {
     *zeta              = \&nan;
     *eta               = \&nan;
     *log               = \&nan;
+    *ilog              = \&nan;
     *log2              = \&nan;
+    *ilog2             = \&nan;
     *log10             = \&nan;
+    *ilog10            = \&nan;
     *lgrt              = \&nan;
+    *lambert_w         = \&nan;
     *mod               = \&nan;
     *imod              = \&nan;
     *fmod              = \&nan;
@@ -189,6 +193,7 @@ package Sidef::Types::Number::Nan {
     *rad               = \&nan;
     *bessel_j          = \&nan;
     *bessel_y          = \&nan;
+    *beta              = \&nan;
 
     sub cmp { }
 
@@ -254,6 +259,10 @@ package Sidef::Types::Number::Nan {
         state $x = Sidef::Types::Number::Complex->new(NAN, NAN);
     }
 
+    sub sin_cos {
+        ((Sidef::Types::Number::Nan::NAN) x 2);
+    }
+
     sub dump {
         state $x = Sidef::Types::String::String->new('NaN');
     }
@@ -320,9 +329,16 @@ package Sidef::Types::Number::Nan {
     *isqrtrem = \&divmod;
     *irootrem = \&divmod;
 
-    sub seed { }
+    sub seed { undef; }
+
     *iseed = \&seed;
 
+    *forperm = \&seed;
+    *forcomb = \&seed;
+
+    *permutations = \&seed;
+    *combinations = \&seed;
+
     {
         no strict 'refs';
         *{__PACKAGE__ . '::' . '/'}   = \&div;
