@@ -59,12 +59,12 @@ package Sidef::Types::Range::RangeNumber {
             if ($repetitions <= $MAX_UINT) {
 
                 if ($repetitions < 0) {
-                    return Sidef::Types::Block::Block->new(code => sub { });
+                    return Sidef::Types::Block::Block->new(code => sub { undef; });
                 }
 
                 return Sidef::Types::Block::Block->new(
                     code => sub {
-                        --$repetitions >= 0 or return;
+                        --$repetitions >= 0 or return undef;
                         $tmp = $i;
                         $i   = $i->add($step);
                         $tmp;
@@ -84,7 +84,7 @@ package Sidef::Types::Range::RangeNumber {
 
         Sidef::Types::Block::Block->new(
             code => sub {
-                ($asc ? $i->le($to) : $i->ge($to)) || return;
+                ($asc ? $i->le($to) : $i->ge($to)) || return undef;
                 $tmp = $i;
                 $i   = $i->add($step);
                 $tmp;
@@ -92,19 +92,23 @@ package Sidef::Types::Range::RangeNumber {
         );
     }
 
+    sub sum_by {
+        my ($self, $arg) = @_;
+        my $sum = Sidef::Types::Number::Number::ZERO;
+
+        my $iter = $self->iter->{code};
+        while (1) {
+            $sum = $sum->add($arg->run($iter->() // last));
+        }
+
+        $sum;
+    }
+
     sub sum {
         my ($self, $arg) = @_;
 
-        # Sum by mapping each value to a block
         if (ref($arg) eq 'Sidef::Types::Block::Block') {
-            my $sum = Sidef::Types::Number::Number::ZERO;
-
-            my $iter = $self->iter->{code};
-            while (1) {
-                $sum = $sum->add($arg->run($iter->() // last));
-            }
-
-            return $sum;
+            goto \&sum_by;
         }
 
         if ($self->{step}->is_one) {
@@ -125,19 +129,24 @@ package Sidef::Types::Range::RangeNumber {
         $sum;
     }
 
+    sub prod_by {
+        my ($self, $arg) = @_;
+
+        my $prod = Sidef::Types::Number::Number::ONE;
+
+        my $iter = $self->iter->{code};
+        while (1) {
+            $prod = $prod->mul($arg->run($iter->() // last));
+        }
+
+        $prod;
+    }
+
     sub prod {
         my ($self, $arg) = @_;
 
-        # Product by mapping each value to a block
         if (ref($arg) eq 'Sidef::Types::Block::Block') {
-            my $prod = Sidef::Types::Number::Number::ONE;
-
-            my $iter = $self->iter->{code};
-            while (1) {
-                $prod = $prod->mul($arg->run($iter->() // last));
-            }
-
-            return $prod;
+            goto \&prod_by;
         }
 
         if (    $self->{step}->is_one
