@@ -104,7 +104,7 @@ package Sidef::Types::Range::RangeString {
 
         Sidef::Types::Block::Block->new(
             code => sub {
-                ($asc ? $i->le($to) : $i->ge($to)) || return;
+                ($asc ? $i->le($to) : $i->ge($to)) || return undef;
                 my $value = $i;
                 $i = $i->add($step);
                 $value->chr;
@@ -114,10 +114,20 @@ package Sidef::Types::Range::RangeString {
 
     sub pick {
         my ($self, $num) = @_;
-        $self->SUPER::pick($num)->map_operator('chr');
+
+        defined($num)
+          ? $self->SUPER::pick($num)->map_operator('chr')
+          : $self->SUPER::pick->chr;
     }
 
-    *rand = \&pick;
+    sub rand {
+        my ($self, $num) = @_;
+        defined($num)
+          ? $self->SUPER::rand($num)->map_operator('chr')
+          : $self->SUPER::rand->chr;
+    }
+
+    *sample = \&rand;
 
     sub dump {
         my ($self) = @_;
