@@ -46,7 +46,7 @@ package Sidef::Types::String::String {
 
     sub div {
         my ($self, $num) = @_;
-        (my $strlen = int(length($$self) / CORE::int($num))) < 1 and return;
+        (my $strlen = CORE::int(length($$self) / CORE::int($num))) < 1 and return $self->chars;
         Sidef::Types::Array::Array->new([map { bless(\$_, __PACKAGE__) } unpack "(a$strlen)*", $$self]);
     }
 
@@ -668,7 +668,7 @@ package Sidef::Types::String::String {
 
     sub chars {
         my ($self) = @_;
-        Sidef::Types::Array::Array->new([map { bless(\$_, __PACKAGE__) } CORE::split(//, CORE::join('', $$self))]);
+        Sidef::Types::Array::Array->new([map { bless(\$_, __PACKAGE__) } CORE::split(//, $$self)]);
     }
 
     sub iter {
@@ -681,7 +681,7 @@ package Sidef::Types::String::String {
 
         Sidef::Types::Block::Block->new(
             code => sub {
-                ++$i <= $end or return;
+                ++$i <= $end or return undef;
                 bless \(my $chr = $chars[$i]), __PACKAGE__;
             }
         );
@@ -758,6 +758,30 @@ package Sidef::Types::String::String {
         Sidef::Types::Glob::File->new(\$string)->open(@rest);
     }
 
+    sub center {
+        my ($self, $size, $char) = @_;
+
+        my $string = $$self;
+
+        $size = defined($size) ? CORE::int($size) : 0;
+        $char = defined($char) ? "$char"          : ' ';
+
+        if (CORE::length($char) > 1) {
+            $char = CORE::substr($char, 0, 1);
+        }
+
+        my $len = CORE::length($string);
+
+        $size <= $len
+          && return $string;
+
+        my $padlen = $size - $len;
+        my $lpad   = $padlen >> 1;
+        my $rpad   = $padlen - $lpad;
+
+        $self->new(($char x $lpad) . $string . ($char x $rpad));
+    }
+
     sub trim {
         my ($self) = @_;
         $self->new(unpack('A*', $$self) =~ s/^\s+//r);
@@ -770,6 +794,7 @@ package Sidef::Types::String::String {
         $self->new($$self =~ s/^\s+//r);
     }
 
+    *ltrim    = \&strip_beg;
     *trim_beg = \&strip_beg;
 
     sub strip_end {
@@ -777,6 +802,7 @@ package Sidef::Types::String::String {
         $self->new(unpack('A*', $$self));
     }
 
+    *rtrim    = \&strip_end;
     *trim_end = \&strip_end;
 
     sub trans {
